/*
 * -----------------------------------------------------------------------------
 *
 * <p><b>License and Copyright: </b>The contents of this file are subject to the
 * Apache License, Version 2.0 (the "License"); you may not use 
 * this file except in compliance with the License. You may obtain a copy of 
 * the License at <a href="http://www.fedora-commons.org/licenses">
 * http://www.fedora-commons.org/licenses.</a></p>
 *
 * <p>Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.</p>
 *
 * <p>The entire file consists of original code.</p>
 * <p>Copyright &copy; 2008 Fedora Commons, Inc.<br />
 * <p>Copyright &copy; 2002-2007 The Rector and Visitors of the University of 
 * Virginia and Cornell University<br /> 
 * All rights reserved.</p>
 *
 * -----------------------------------------------------------------------------
 */
package fedora.common.policy;

import com.sun.xacml.attr.StringAttribute;

public class ActionNamespace extends XacmlNamespace { 
	
	// Properties
	public final XacmlName ID;	
	public final XacmlName API;
	public final XacmlName CONTEXT_ID;

    // Values of API
	public final XacmlName APIM;
	public final XacmlName APIA;
	
    // Values of ID   
	public final XacmlName ADD_DATASTREAM;	
	public final XacmlName ADD_DISSEMINATOR;	
	public final XacmlName ADMIN_PING;
	public final XacmlName EXPORT;	
	public final XacmlName GET_DATASTREAM;	
	public final XacmlName GET_DATASTREAM_HISTORY;	
	public final XacmlName GET_DATASTREAMS;	
	public final XacmlName GET_DISSEMINATOR;
	public final XacmlName GET_DISSEMINATORS;	
	public final XacmlName GET_DISSEMINATOR_HISTORY;	
	public final XacmlName GET_NEXT_PID;
	public final XacmlName GET_OBJECT_PROPERTIES;	
	public final XacmlName GET_OBJECT_XML;	
	public final XacmlName INGEST;
	public final XacmlName MODIFY_DATASTREAM_BY_REFERENCE;	
	public final XacmlName MODIFY_DATASTREAM_BY_VALUE;
	public final XacmlName MODIFY_DISSEMINATOR;		
	public final XacmlName MODIFY_OBJECT;
	public final XacmlName PURGE_OBJECT;
	public final XacmlName PURGE_DATASTREAM;
	public final XacmlName PURGE_DISSEMINATOR;	
	public final XacmlName SET_DATASTREAM_STATE;	
	public final XacmlName SET_DISSEMINATOR_STATE;	
	public final XacmlName DESCRIBE_REPOSITORY;	
	public final XacmlName FIND_OBJECTS;
	public final XacmlName RI_FIND_OBJECTS;	
	public final XacmlName GET_DATASTREAM_DISSEMINATION;	
	public final XacmlName GET_DISSEMINATION;	
	public final XacmlName GET_OBJECT_HISTORY;	
	public final XacmlName GET_OBJECT_PROFILE;	
	public final XacmlName LIST_DATASTREAMS;	
	public final XacmlName LIST_METHODS;		
	public final XacmlName LIST_OBJECT_IN_FIELD_SEARCH_RESULTS;
	public final XacmlName LIST_OBJECT_IN_RESOURCE_INDEX_RESULTS;
	public final XacmlName SERVER_SHUTDOWN;
	public final XacmlName SERVER_STATUS;
	public final XacmlName OAI;	
	public final XacmlName UPLOAD;	
	public final XacmlName INTERNAL_DSSTATE;
	public final XacmlName RESOLVE_DATASTREAM;
	public final XacmlName RELOAD_POLICIES;	
	public final XacmlName SET_DATASTREAM_VERSIONABLE;
	public final XacmlName COMPARE_DATASTREAM_CHECKSUM;

    private ActionNamespace(XacmlNamespace parent, String localName) {
    	super(parent, localName);
    	API = addName(new XacmlName(this, "api", StringAttribute.identifier));
    	APIM               = addName(new XacmlName(this, "api-m"));
    	APIA               = addName(new XacmlName(this, "api-a"));

    	ID = addName(new XacmlName(this, "id", StringAttribute.identifier));
    	// derived from respective Java methods in Access.java or Management.java    	
    	ADD_DATASTREAM               = addName(new XacmlName(this, "id-addDatastream"));	
    	ADD_DISSEMINATOR               = addName(new XacmlName(this, "id-addDisseminator"));	
    	ADMIN_PING               = addName(new XacmlName(this, "id-adminPing")); 
    	EXPORT               = addName(new XacmlName(this, "id-export"));    	
    	GET_DATASTREAM               = addName(new XacmlName(this, "id-getDatastream"));	
    	GET_DATASTREAM_HISTORY               = addName(new XacmlName(this, "id-getDatastreamHistory"));	
    	GET_DATASTREAMS               = addName(new XacmlName(this, "id-getDatastreams"));	
    	GET_DISSEMINATOR               = addName(new XacmlName(this, "id-getDisseminator"));
    	GET_DISSEMINATORS               = addName(new XacmlName(this, "id-getDisseminators"));	
    	GET_DISSEMINATOR_HISTORY               = addName(new XacmlName(this, "id-getDisseminatorHistory"));	
    	GET_NEXT_PID               = addName(new XacmlName(this, "id-getNextPid"));
    	GET_OBJECT_PROPERTIES               = addName(new XacmlName(this, "id-getObjectProperties"));	
    	GET_OBJECT_XML               = addName(new XacmlName(this, "id-getObjectXML"));	
    	INGEST               = addName(new XacmlName(this, "id-ingest"));
    	MODIFY_DATASTREAM_BY_REFERENCE               = addName(new XacmlName(this, "id-modifyDatastreamByReference"));	
    	MODIFY_DATASTREAM_BY_VALUE               = addName(new XacmlName(this, "id-modifyDatastreamByValue"));
    	MODIFY_DISSEMINATOR               = addName(new XacmlName(this, "id-modifyDisseminator"));		
    	MODIFY_OBJECT               = addName(new XacmlName(this, "id-modifyObject"));
    	PURGE_OBJECT               = addName(new XacmlName(this, "id-purgeObject"));
    	PURGE_DATASTREAM               = addName(new XacmlName(this, "id-purgeDatastream"));
    	PURGE_DISSEMINATOR               = addName(new XacmlName(this, "id-purgeDisseminator"));	
    	SET_DATASTREAM_STATE               = addName(new XacmlName(this, "id-setDatastreamState"));	
    	SET_DISSEMINATOR_STATE               = addName(new XacmlName(this, "id-setDisseminatorState"));	
    	SET_DATASTREAM_VERSIONABLE               = addName(new XacmlName(this, "id-setDatastreamVersionable"));
    	COMPARE_DATASTREAM_CHECKSUM               = addName(new XacmlName(this, "id-compareDatastreamChecksum"));
    	DESCRIBE_REPOSITORY               = addName(new XacmlName(this, "id-describeRepository"));	
    	FIND_OBJECTS               = addName(new XacmlName(this, "id-findObjects"));	
    	RI_FIND_OBJECTS               = addName(new XacmlName(this, "id-riFindObjects"));	
    	GET_DATASTREAM_DISSEMINATION               = addName(new XacmlName(this, "id-getDatastreamDissemination"));	
    	GET_DISSEMINATION               = addName(new XacmlName(this, "id-getDissemination"));	
    	GET_OBJECT_HISTORY               = addName(new XacmlName(this, "id-getObjectHistory"));	
    	GET_OBJECT_PROFILE               = addName(new XacmlName(this, "id-getObjectProfile"));	
    	LIST_DATASTREAMS               = addName(new XacmlName(this, "id-listDatastreams"));	
    	LIST_METHODS               = addName(new XacmlName(this, "id-listMethods"));		
    	LIST_OBJECT_IN_FIELD_SEARCH_RESULTS               = addName(new XacmlName(this, "id-listObjectInFieldSearchResults"));
    	LIST_OBJECT_IN_RESOURCE_INDEX_RESULTS               = addName(new XacmlName(this, "id-listObjectInResourceIndexResults"));
    	SERVER_SHUTDOWN = addName(new XacmlName(this, "id-serverShutdown")); 
    	SERVER_STATUS = addName(new XacmlName(this, "id-serverStatus"));    
    	OAI = addName(new XacmlName(this, "id-oai"));     	
    	UPLOAD = addName(new XacmlName(this, "id-upload"));     	
    	INTERNAL_DSSTATE = addName(new XacmlName(this, "id-dsstate"));
    	RESOLVE_DATASTREAM = addName(new XacmlName(this, "id-resolveDatastream"));    	
    	RELOAD_POLICIES = addName(new XacmlName(this, "id-reloadPolicies"));    	
    	CONTEXT_ID = addName(new XacmlName(this, "contextId", StringAttribute.identifier)); //internal callback support
    	// Values of CONTEXT_ID are sequential numerals, hence not enumerated here.
    	
    }

	public static ActionNamespace onlyInstance = new ActionNamespace(Release2_1Namespace.getInstance(), "action");
	
	public static final ActionNamespace getInstance() {
		return onlyInstance;
	}

}
