/*
 * -----------------------------------------------------------------------------
 *
 * <p><b>License and Copyright: </b>The contents of this file are subject to the
 * Apache License, Version 2.0 (the "License"); you may not use 
 * this file except in compliance with the License. You may obtain a copy of 
 * the License at <a href="http://www.fedora-commons.org/licenses">
 * http://www.fedora-commons.org/licenses.</a></p>
 *
 * <p>Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.</p>
 *
 * <p>The entire file consists of original code.</p>
 * <p>Copyright &copy; 2008 Fedora Commons, Inc.<br />
 * <p>Copyright &copy; 2002-2007 The Rector and Visitors of the University of 
 * Virginia and Cornell University<br /> 
 * All rights reserved.</p>
 *
 * -----------------------------------------------------------------------------
 */
package fedora.server;

import java.util.Iterator;

/**
 * A <code>Context</code> used for recovery.
 *
 * This Context provides access to attributes in the recovery namespace.
 *
 * @see fedora.common.Constants#RECOVERY
 */
public interface RecoveryContext extends Context {

    /**
     * Get the names of all <em>recovery</em> attributes whose values are 
     * defined in this context.
     */
    public Iterator getRecoveryNames();

    /**
     * Get the first value for a <em>recovery</em> attribute, or 
     * <code>null</code> if no such value exists in this context.
     */
    public String getRecoveryValue(String attribute);

    /**
     * Get all values for a <em>recovery</em> attribute, or an empty array 
     * if no values exist for the attribute in this context.
     */
    public String[] getRecoveryValues(String attribute);

}
