/*
 * -----------------------------------------------------------------------------
 *
 * <p><b>License and Copyright: </b>The contents of this file are subject to the
 * Apache License, Version 2.0 (the "License"); you may not use 
 * this file except in compliance with the License. You may obtain a copy of 
 * the License at <a href="http://www.fedora-commons.org/licenses">
 * http://www.fedora-commons.org/licenses.</a></p>
 *
 * <p>Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.</p>
 *
 * <p>The entire file consists of original code.</p>
 * <p>Copyright &copy; 2008 Fedora Commons, Inc.<br />
 * <p>Copyright &copy; 2002-2007 The Rector and Visitors of the University of 
 * Virginia and Cornell University<br /> 
 * All rights reserved.</p>
 *
 * -----------------------------------------------------------------------------
 */
package fedora.server.errors;

/**
 * <p><b>Title: </b>ObjectDependencyException.java</p>
 * <p><b>Description: </b>Signals that an object has one or more related objects that
 * depend on it. For example, a behavior definition or behavior mechanism
 * object can be shared by multiple objects. Any data objects that use a
 * specific behavior definition or behavior mechanism object "depend" on
 * those objects. To remove a dependent object, you must first remove all
 * related objects.</p>
 *
 * @author rlw@virginia.edu
 * @version $Id: ObjectDependencyException.java 5162 2006-10-25 00:49:06Z eddie $
 */
public class ObjectDependencyException
        extends StorageException {

	private static final long serialVersionUID = 1L;
	
    /**
     * Creates an ObjectDependencyException.
     *
     * @param message An informative message explaining what happened and
     *                (possibly) how to fix it.
     */
    public ObjectDependencyException(String message) {
        super(message);
    }

}