/*
 * -----------------------------------------------------------------------------
 *
 * <p><b>License and Copyright: </b>The contents of this file are subject to the
 * Apache License, Version 2.0 (the "License"); you may not use 
 * this file except in compliance with the License. You may obtain a copy of 
 * the License at <a href="http://www.fedora-commons.org/licenses">
 * http://www.fedora-commons.org/licenses.</a></p>
 *
 * <p>Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.</p>
 *
 * <p>The entire file consists of original code.</p>
 * <p>Copyright &copy; 2008 Fedora Commons, Inc.<br />
 * <p>Copyright &copy; 2002-2007 The Rector and Visitors of the University of 
 * Virginia and Cornell University<br /> 
 * All rights reserved.</p>
 *
 * -----------------------------------------------------------------------------
 */
package fedora.server.management;

import java.io.IOException;
import fedora.common.PID;

/**
 * Interface for generating Fedora PIDs.
 *
 * @author cwilper@cs.cornell.edu (first version, Paul Charlton)
 * @version $Id: PIDGenerator.java 4146 2005-05-19 19:13:37Z eddie $
 */
public interface PIDGenerator {

    /**
     * Generate a new pid that is guaranteed to be unique, within the
     * given namespace.
     * 
     * @param namespace
     * @return the PID
     * @throws IOException
     */
    public PID generatePID(String namespace) throws IOException;

    /**
     * Get the last pid that was generated.
     * @return the last PID that was generated
     * @throws IOException
     */
    public PID getLastPID() throws IOException;

    /**
     * Cause the given PID to never be generated by the PID generator.
     * @param pid
     * @throws IOException
     */
    public void neverGeneratePID(String pid) throws IOException;
}
