/*
 * -----------------------------------------------------------------------------
 *
 * <p><b>License and Copyright: </b>The contents of this file are subject to the
 * Apache License, Version 2.0 (the "License"); you may not use 
 * this file except in compliance with the License. You may obtain a copy of 
 * the License at <a href="http://www.fedora-commons.org/licenses">
 * http://www.fedora-commons.org/licenses.</a></p>
 *
 * <p>Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.</p>
 *
 * <p>The entire file consists of original code.</p>
 * <p>Copyright &copy; 2008 Fedora Commons, Inc.<br />
 * <p>Copyright &copy; 2002-2007 The Rector and Visitors of the University of 
 * Virginia and Cornell University<br /> 
 * All rights reserved.</p>
 *
 * -----------------------------------------------------------------------------
 */
package fedora.server.resourceIndex;

import java.util.Set;

import fedora.server.errors.ResourceIndexException;

/**
 * Provides key information about known service method implementations.
 *
 * @author cwilper@cs.cornell.edu
 */
public interface MethodInfoProvider {

    /**
     * Get the <code>MethodInfo</code> for each method of the given 
     * behavior mechanism.
     *
     * @param bMechPID the behavior mechanism pid.
     * @return A set with one item for each implemented method.
     * @throws ResourceIndexException if no such behavior mechanism exists
     *         or the information can't be read for any other reason.
     */
    Set<MethodInfo> getMethodInfo(String bMechPID)
            throws ResourceIndexException;

}