/*
 * -----------------------------------------------------------------------------
 *
 * <p><b>License and Copyright: </b>The contents of this file are subject to the
 * Apache License, Version 2.0 (the "License"); you may not use 
 * this file except in compliance with the License. You may obtain a copy of 
 * the License at <a href="http://www.fedora-commons.org/licenses">
 * http://www.fedora-commons.org/licenses.</a></p>
 *
 * <p>Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.</p>
 *
 * <p>The entire file consists of original code.</p>
 * <p>Copyright &copy; 2008 Fedora Commons, Inc.<br />
 * <p>Copyright &copy; 2002-2007 The Rector and Visitors of the University of 
 * Virginia and Cornell University<br /> 
 * All rights reserved.</p>
 *
 * -----------------------------------------------------------------------------
 */
package fedora.server.resourceIndex;

import fedora.server.errors.ResourceIndexException;
import fedora.server.storage.BDefReader;
import fedora.server.storage.BMechReader;

/**
 * Stores and provides key information about known service method 
 * implementations.
 *
 * @author cwilper@cs.cornell.edu
 */
public interface MethodInfoStore extends MethodInfoProvider {

    /**
     * Add or replace method information about the given behavior definition.
     *
     * A behavior definition defines a set of method names and the runtime
     * parameters for each.
     *
     * @param reader the behavior definition.
     * @throws ResourceIndexException if any error occurs.
     */
    void putBDefInfo(BDefReader reader) throws ResourceIndexException;

    /**
     * Add or replace method information about the given behavior mechanism.
     *
     * For each method it implements, a behavior mechanism defines a
     * set of datastream binding keys and a set of possible return types.
     *
     * @param reader the behavior mechanism.
     * @throws ResourceIndexException if any error occurs.
     */
    void putBMechInfo(BMechReader reader) throws ResourceIndexException;

    /**
     * Delete method information about the given behavior definition.
     *
     * @param bDefPID the pid of the behavior definition whose information
     *                should be deleted.
     * @throws ResourceIndexException if any error occurs.
     */
    void deleteBDefInfo(String bDefPID) throws ResourceIndexException;

    /**
     * Delete method information about the given behavior mechanism.
     *
     * @param bDefPID the pid of the behavior mechanism whose information
     *                should be deleted.
     * @throws ResourceIndexException if any error occurs.
     */
    void deleteBMechInfo(String bMechPID) throws ResourceIndexException;

}