/*
 * -----------------------------------------------------------------------------
 *
 * <p><b>License and Copyright: </b>The contents of this file are subject to the
 * Apache License, Version 2.0 (the "License"); you may not use 
 * this file except in compliance with the License. You may obtain a copy of 
 * the License at <a href="http://www.fedora-commons.org/licenses">
 * http://www.fedora-commons.org/licenses.</a></p>
 *
 * <p>Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.</p>
 *
 * <p>The entire file consists of original code.</p>
 * <p>Copyright &copy; 2008 Fedora Commons, Inc.<br />
 * <p>Copyright &copy; 2002-2007 The Rector and Visitors of the University of 
 * Virginia and Cornell University<br /> 
 * All rights reserved.</p>
 *
 * -----------------------------------------------------------------------------
 */
package fedora.server.resourceIndex;

import java.util.Set;

import org.jrdf.graph.Triple;

import fedora.server.errors.ResourceIndexException;
import fedora.server.storage.BDefReader;
import fedora.server.storage.BMechReader;
import fedora.server.storage.DOReader;

/**
 * Generates RDF triples for Fedora objects.
 *
 * @author cwilper@cs.cornell.edu
 */
public interface TripleGenerator {

    /**
     * Get the triples for the given Fedora behavior definition object.
     *
     * @param reader the behavior definition object.
     * @return the set of triples.
     */
    Set<Triple> getTriplesForBDef(BDefReader reader)
            throws ResourceIndexException;

    /**
     * Get the triples for the given Fedora behavior mechanism object.
     *
     * @param reader the behavior mechanism object.
     * @return the set of triples.
     */
    Set<Triple> getTriplesForBMech(BMechReader reader)
            throws ResourceIndexException;

    /**
     * Get the triples for the given Fedora data object.
     *
     * @param reader the data object.
     * @return the set of triples.
     */
    Set<Triple> getTriplesForDataObject(DOReader reader)
            throws ResourceIndexException;

}
