/*
 * -----------------------------------------------------------------------------
 *
 * <p><b>License and Copyright: </b>The contents of this file are subject to the
 * Apache License, Version 2.0 (the "License"); you may not use 
 * this file except in compliance with the License. You may obtain a copy of 
 * the License at <a href="http://www.fedora-commons.org/licenses">
 * http://www.fedora-commons.org/licenses.</a></p>
 *
 * <p>Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.</p>
 *
 * <p>The entire file consists of original code.</p>
 * <p>Copyright &copy; 2008 Fedora Commons, Inc.<br />
 * <p>Copyright &copy; 2002-2007 The Rector and Visitors of the University of 
 * Virginia and Cornell University<br /> 
 * All rights reserved.</p>
 *
 * -----------------------------------------------------------------------------
 */
package fedora.server.security;

import fedora.server.errors.BackendSecurityParserException;
import fedora.server.security.BackendSecuritySpec;

/**
*
* <p><b>Title:</b> BackendSecurity.java</p>
* <p><b>Description:</b> An interface for accessing backend service security info.</p>
*
* @author rlw@virginia.edu
* @version $Id: BackendSecurity.java 4164 2005-05-27 14:42:01Z rlw $
*/
public interface BackendSecurity {
    
    /**
     * <p>Gets the current instance of BackendSecuritySpec.</p>
     * 
     * @return Current instance of backendSecuritySpec.
     */
    public BackendSecuritySpec getBackendSecuritySpec();
    
    /**
     * <p>Sets the current instance of BackendSecuritySpec by parsing the
     * beSecurity configuration file.</p>
     * 
     * @throws BackendSecurityParserException If an error occurs in parsing the
     *         beSecurity configuration file.
     */
    public void setBackendSecuritySpec() throws BackendSecurityParserException;
    
    /**
     * <p>Parses the beSecurity configuration file and stores the results in an
     * instance of BackendSecuritySpec.</p>
     * 
     * @return An instance of BackendSecuritySpec.
     * @throws BackendSecurityParserException If an error occursin parsing the
     *         beSecurity configuration file.
     */
    public BackendSecuritySpec parseBeSecurity() throws BackendSecurityParserException;
    
    /**
     * <p>Reloads the backend service security info by reparsing the beSecurity
     * configuration file and storing results in an instance of BackendSecuritySpec.</p>
     * 
     * @return An instance of BackendSecuritySpec.
     * @throws BackendSecurityParserException If an error occurs in trying to reparse the
     *         beSecurity configuration file.
     */
    public BackendSecuritySpec reloadBeSecurity() throws BackendSecurityParserException;

}
