/*
 * -----------------------------------------------------------------------------
 *
 * <p><b>License and Copyright: </b>The contents of this file are subject to the
 * Apache License, Version 2.0 (the "License"); you may not use 
 * this file except in compliance with the License. You may obtain a copy of 
 * the License at <a href="http://www.fedora-commons.org/licenses">
 * http://www.fedora-commons.org/licenses.</a></p>
 *
 * <p>Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.</p>
 *
 * <p>The entire file consists of original code.</p>
 * <p>Copyright &copy; 2008 Fedora Commons, Inc.<br />
 * <p>Copyright &copy; 2002-2007 The Rector and Visitors of the University of 
 * Virginia and Cornell University<br /> 
 * All rights reserved.</p>
 *
 * -----------------------------------------------------------------------------
 */
package fedora.server.security.servletfilters;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import java.util.Enumeration;
import java.util.Map;
import java.util.Hashtable;
import java.util.Hashtable;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.security.Principal;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import java.lang.reflect.Method;
import java.lang.NoSuchMethodException;
import java.io.IOException;
import org.apache.commons.codec.binary.Base64;

import fedora.server.security.servletfilters.pubcookie.FilterPubcookie;


/** 
 *  @author Bill Niebel (niebel@virginia.edu)
 */
public class Base {
    protected static Log log = LogFactory.getLog(Base.class);
    
    protected static final String[] StringArrayPrototype = new String[0];
    
    private static final String INTO = " . . .";
    private static final String OUTOF = ". . . ";
    
    protected String getClassName() {
    	String classname = this.getClass().getName();
    	String[] parts = classname.split("\\.");
    	if (parts.length > 0) { 
    		classname = parts[parts.length - 1];
    	}
    	return classname;
    }
    
    public final String enter(String method) {
    	return getClassName() + 
    	"." + method + " " + INTO;
    }

    public final String exit(String method) {
    	return getClassName() + 
    	"." + OUTOF + " " + method;
    }
    
    public final String enterExit(String method) {
    	return getClassName() + 
    	"." + OUTOF + method + INTO;
    }
    
    public final String passFail(String method, String test, String result) {
		return getClassName() + 
		"." + method + ": " + result + " " + test + " test";
    }

    public final String pass(String method, String test) {
		return passFail(method, test, "passed");
	}
	
    public final String fail(String method, String test) {
		return passFail(method, test, "failed");
	}

    public final String format(String method, String msg) {
		return getClassName() + 
		"." + method + ": " + msg;
	}
    
    public final String format(String method, String msg, String name) {
		return format(method, msg, name, null);
	}
    
    public final String format(String method, String msg, String name, String value) {
		return getClassName() + 
		"." + method + ": " + ((msg==null) ? "" : msg + " ") + name + "==" + ((value==null) ? "" : value);
	}
    

    public final void showThrowable(Throwable th, Log log, String msg) {
		if (log.isErrorEnabled()) {
			if (msg != null) log.error(msg);
			log.error(th);
			log.error(th.getMessage());
			if (th.getCause() != null) log.error(th.getCause().getMessage());
			th.printStackTrace();
		}    	
    }
	
    public static final boolean booleanValue(String string) throws Exception {
    	if (Boolean.TRUE.toString().equals(string) ||  Boolean.FALSE.toString().equals(string)) {
    		return (new Boolean(string)).booleanValue();
    	} else {
    		throw new Exception("does not represent a boolean");
    	}
    }

    protected boolean initErrors = false;
    
    protected void initThisSubclass(String key, String value) {
    	log.debug("AFB.iTS");
    	String method = "initThisSubclass() "; if (log.isDebugEnabled()) log.debug(enter(method));
    	initErrors = true;
    	if (log.isErrorEnabled()) log.error(format(method, "unknown parameter",  key, value));
		if (log.isDebugEnabled()) log.debug(exit(method)); 
	}
    
}
