/*
 * -----------------------------------------------------------------------------
 *
 * <p><b>License and Copyright: </b>The contents of this file are subject to the
 * Apache License, Version 2.0 (the "License"); you may not use 
 * this file except in compliance with the License. You may obtain a copy of 
 * the License at <a href="http://www.fedora-commons.org/licenses">
 * http://www.fedora-commons.org/licenses.</a></p>
 *
 * <p>Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.</p>
 *
 * <p>The entire file consists of original code.</p>
 * <p>Copyright &copy; 2008 Fedora Commons, Inc.<br />
 * <p>Copyright &copy; 2002-2007 The Rector and Visitors of the University of 
 * Virginia and Cornell University<br /> 
 * All rights reserved.</p>
 *
 * -----------------------------------------------------------------------------
 */
package fedora.utilities.install.container;

import java.io.File;

import fedora.utilities.install.InstallOptions;

/**
 * Options for the Fedora web.xml file.
 * 
 * @author Edwin Shin
 */
public class WebXMLOptions {
	public boolean apiaAuth;
	public boolean apiaSSL;
	public boolean apimSSL;
	public File fedoraHome;
	
	public WebXMLOptions() {}
	
	public WebXMLOptions(InstallOptions installOptions) {
		apiaAuth = installOptions.getBooleanValue(InstallOptions.APIA_AUTH_REQUIRED, false);
		apiaSSL = installOptions.getBooleanValue(InstallOptions.APIA_SSL_REQUIRED, false);
		apimSSL = installOptions.getBooleanValue(InstallOptions.APIM_SSL_REQUIRED, false);
		fedoraHome = new File(installOptions.getValue(InstallOptions.FEDORA_HOME));
	}

	public boolean isApiaAuth() {
		return apiaAuth;
	}

	public void setApiaAuth(boolean apiaAuth) {
		this.apiaAuth = apiaAuth;
	}

	public boolean isApiaSSL() {
		return apiaSSL;
	}

	public void setApiaSSL(boolean apiaSSL) {
		this.apiaSSL = apiaSSL;
	}

	public boolean isApimSSL() {
		return apimSSL;
	}

	public void setApimSSL(boolean apimSSL) {
		this.apimSSL = apimSSL;
	}

	public File getFedoraHome() {
		return fedoraHome;
	}

	public void setFedoraHome(File fedoraHome) {
		this.fedoraHome = fedoraHome;
	}
}
